# The MIT License (MIT)
#
# Copyright (c) 2017 Tony DiCola for Adafruit Industries
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# """
# `adafruit_lsm9ds0`
# ====================================================
# CircuitPython module for the LSM9DS0 accelerometer, magnetometer, gyroscope.
# Based on the driver from:
# https://github.com/adafruit/Adafruit_LSM9DS0
# See examples/simpletest.py for a demo of the usage.
# * Author(s): Tony DiCola, Gaetan Belliere
# Implementation Notes
# --------------------
# **Hardware:**
# * Adafruit `9-DOF Accel/Mag/Gyro+Temp Breakout Board - LSM9DS0
#   <https://www.adafruit.com/product/2021>`_ (Product ID: 2021)
# * FLORA `9-DOF Accelerometer/Gyroscope/Magnetometer - LSM9DS0
#   <https://www.adafruit.com/product/2020>`_ (Product ID: 2020)
# **Software and Dependencies:**
# * Adafruit CircuitPython firmware for the ESP8622 and M0-based boards:
#   https://github.com/adafruit/circuitpython/releases
# * Adafruit's Bus Device library: https://github.com/adafruit/Adafruit_CircuitPython_BusDevice
# """

####################################################################
#                   IMPORT
####################################################################

from machine import SPI
from machine import Pin
import binascii
from micropython import const

####################################################################
#                   REGISTRES
####################################################################

# Internal constants and register values:
# pylint: disable=bad-whitespace
_LSM9DS0_ADDRESS_ACCELMAG           = const(0x1D)  # 3B >> 1 = 7bit default
_LSM9DS0_ADDRESS_GYRO               = const(0x6B)  # D6 >> 1 = 7bit default
_LSM9DS0_XM_ID                      = const(0b01001001)
_LSM9DS0_G_ID                       = const(0b11010100)
_LSM9DS0_ACCEL_MG_LSB_2G            = 0.061
_LSM9DS0_ACCEL_MG_LSB_4G            = 0.122
_LSM9DS0_ACCEL_MG_LSB_6G            = 0.183
_LSM9DS0_ACCEL_MG_LSB_8G            = 0.244
_LSM9DS0_ACCEL_MG_LSB_16G           = 0.732 # Is this right? Was expecting 0.488
_LSM9DS0_MAG_MGAUSS_2GAUSS          = 0.08
_LSM9DS0_MAG_MGAUSS_4GAUSS          = 0.16
_LSM9DS0_MAG_MGAUSS_8GAUSS          = 0.32
_LSM9DS0_MAG_MGAUSS_12GAUSS         = 0.48
_LSM9DS0_GYRO_DPS_DIGIT_245DPS      = 0.00875
_LSM9DS0_GYRO_DPS_DIGIT_500DPS      = 0.01750
_LSM9DS0_GYRO_DPS_DIGIT_2000DPS     = 0.07000
_LSM9DS0_TEMP_LSB_DEGREE_CELSIUS    = 8  # 1°C = 8, 25° = 200, etc.
_LSM9DS0_REGISTER_WHO_AM_I          = const(0x0F)
_LSM9DS0_REGISTER_CTRL_REG1_G       = const(0x20)
_LSM9DS0_REGISTER_CTRL_REG3_G       = const(0x22)
_LSM9DS0_REGISTER_CTRL_REG4_G       = const(0x23)
_LSM9DS0_REGISTER_OUT_X_L_G         = const(0x28)
_LSM9DS0_REGISTER_OUT_X_H_G         = const(0x29)
_LSM9DS0_REGISTER_OUT_Y_L_G         = const(0x2A)
_LSM9DS0_REGISTER_OUT_Y_H_G         = const(0x2B)
_LSM9DS0_REGISTER_OUT_Z_L_G         = const(0x2C)
_LSM9DS0_REGISTER_OUT_Z_H_G         = const(0x2D)
_LSM9DS0_REGISTER_TEMP_OUT_L_XM     = const(0x05)
_LSM9DS0_REGISTER_TEMP_OUT_H_XM     = const(0x06)
_LSM9DS0_REGISTER_STATUS_REG_M      = const(0x07)
_LSM9DS0_REGISTER_OUT_X_L_M         = const(0x08)
_LSM9DS0_REGISTER_OUT_X_H_M         = const(0x09)
_LSM9DS0_REGISTER_OUT_Y_L_M         = const(0x0A)
_LSM9DS0_REGISTER_OUT_Y_H_M         = const(0x0B)
_LSM9DS0_REGISTER_OUT_Z_L_M         = const(0x0C)
_LSM9DS0_REGISTER_OUT_Z_H_M         = const(0x0D)
_LSM9DS0_REGISTER_WHO_AM_I_XM       = const(0x0F)
_LSM9DS0_REGISTER_INT_CTRL_REG_M    = const(0x12)
_LSM9DS0_REGISTER_INT_SRC_REG_M     = const(0x13)
_LSM9DS0_REGISTER_CTRL_REG1_XM      = const(0x20)
_LSM9DS0_REGISTER_CTRL_REG2_XM      = const(0x21)
_LSM9DS0_REGISTER_CTRL_REG5_XM      = const(0x24)
_LSM9DS0_REGISTER_CTRL_REG6_XM      = const(0x25)
_LSM9DS0_REGISTER_CTRL_REG7_XM      = const(0x26)
_LSM9DS0_REGISTER_OUT_X_L_A         = const(0x28)
_LSM9DS0_REGISTER_OUT_X_H_A         = const(0x29)
_LSM9DS0_REGISTER_OUT_Y_L_A         = const(0x2A)
_LSM9DS0_REGISTER_OUT_Y_H_A         = const(0x2B)
_LSM9DS0_REGISTER_OUT_Z_L_A         = const(0x2C)
_LSM9DS0_REGISTER_OUT_Z_H_A         = const(0x2D)
_GYROTYPE                           = True
_XMTYPE                             = False
_SENSORS_GRAVITY_STANDARD           = 9.80665

# User facing constants/module globals.
ACCELRANGE_2G                = (0b000 << 3)
ACCELRANGE_4G                = (0b001 << 3)
ACCELRANGE_6G                = (0b010 << 3)
ACCELRANGE_8G                = (0b011 << 3)
ACCELRANGE_16G               = (0b100 << 3)
MAGGAIN_2GAUSS               = (0b00 << 5)  # +/- 2 gauss
MAGGAIN_4GAUSS               = (0b01 << 5)  # +/- 4 gauss
MAGGAIN_8GAUSS               = (0b10 << 5)  # +/- 8 gauss
MAGGAIN_12GAUSS              = (0b11 << 5)  # +/- 12 gauss
GYROSCALE_245DPS             = (0b00 << 4)  # +/- 245 degrees per second rotation
GYROSCALE_500DPS             = (0b01 << 4)  # +/- 500 degrees per second rotation
GYROSCALE_2000DPS            = (0b10 << 4)  # +/- 2000 degrees per second rotation
# pylint: enable=bad-whitespace

####################################################################
#                   LIRE OU ECRIRE DANS REGISTRE SPI
####################################################################

#ecrire un byte dans un registre en liaison SPI
def writeRegistre(spi,cs,addr,data):
    cs.value(0)
    spi.write(addr)
    spi.write(data)
    cs.value(1)
    return 0

#fonction pour lire un  byte dans 1 registre en liaison SPI
def readRegistre(spi,cs,addr):
    cs.value(0)
    spi.write(0x80 | addr)
    buf=spi.read(1,write=0x00)
    cs.value(1)
    return buf

####################################################################
#                   PARAMETRAGE ET TESTS
####################################################################

#initialise connexion SPI
def instancier_spi():
    spi = SPI(0, mode=SPI.MASTER, baudrate=100000, polarity=0, phase=0, firstbit=SPI.MSB)
    #polarity = 0 : etat de repos 0
    #phase = 0 : front montant de clk
    # this uses the SPI default pins for CLK, MOSI and MISO (``P10``, ``P11`` and ``P14``)
    return spi

#instancie les chip select pour informer le capteur associe qu'on s'adresse a lui
def instancier_cs(num):
    cs = Pin(num, mode=Pin.OUT)
    cs.value(1)
    return cs

#Permet de configurer le capteur pour qu'il renvoie des données
#au format de notre choix
def parametrer_capteur(spi,csGyro,csAccel):
    #parametres gyro
    writeRegistre(spi,csGyro,_LSM9DS0_REGISTER_CTRL_REG1_G,0b00001111) #X,Y,Z enabled and power ON
    writeRegistre(spi,csGyro,_LSM9DS0_REGISTER_CTRL_REG4_G,GYROSCALE_2000DPS)
    #parametres accelerometre et magnetometre
    writeRegistre(spi,csAccel,_LSM9DS0_REGISTER_CTRL_REG1_XM,0b01100111) #100Hz, continuous, X,Y,Z enabled
    writeRegistre(spi,csAccel,_LSM9DS0_REGISTER_CTRL_REG2_XM,ACCELRANGE_4G)
    writeRegistre(spi,csAccel,_LSM9DS0_REGISTER_CTRL_REG5_XM,0b01110100) #100Hz, high resolution magneto, temp disabled
    writeRegistre(spi,csAccel,_LSM9DS0_REGISTER_CTRL_REG6_XM,MAGGAIN_4GAUSS)
    writeRegistre(spi,csAccel,_LSM9DS0_REGISTER_CTRL_REG7_XM,0b00000000)
    return 0

#teste la bonne communication avec 1 capteur
def test_capteur(spi,cs):
    buf=bytearray(1)
    buf=readRegistre(spi,cs,_LSM9DS0_REGISTER_WHO_AM_I)
    WhoAmI=int.from_bytes(buf, 0)
    return WhoAmI==_LSM9DS0_G_ID or WhoAmI==_LSM9DS0_XM_ID

#Teste tous les capteurs du module en appelant fonction precedente
def test_capteurs(spi,csGyro,csAccel):
    if test_capteur(spi,csGyro)!=True:
        print("Erreur gyroscope")

    if test_capteur(spi,csAccel)!=True:
        print("Erreur Accel")
    return 0

####################################################################
#                   LECTURE DES DONNEES
####################################################################

#lecture donnees gyroscope
def readGyroscope(spi,cs):
    bufL=readRegistre(spi,cs,_LSM9DS0_REGISTER_OUT_X_L_G)
    bufH=readRegistre(spi,cs,_LSM9DS0_REGISTER_OUT_X_H_G)
    x=convertirData( int.from_bytes(bufH,"big"), int.from_bytes(bufL,"big"))

    bufL=readRegistre(spi,cs,_LSM9DS0_REGISTER_OUT_Y_L_G)
    bufH=readRegistre(spi,cs,_LSM9DS0_REGISTER_OUT_Y_H_G)
    y=convertirData( int.from_bytes(bufH,"big"), int.from_bytes(bufL,"big"))

    bufL=readRegistre(spi,cs,_LSM9DS0_REGISTER_OUT_Z_L_G)
    bufH=readRegistre(spi,cs,_LSM9DS0_REGISTER_OUT_Z_H_G)
    z=convertirData( int.from_bytes(bufH,"big"), int.from_bytes(bufL,"big"))

    return x,y,z

#lecture donnees accelerometre
def readAccel(spi,cs):
    bufL=readRegistre(spi,cs,_LSM9DS0_REGISTER_OUT_X_L_A)
    bufH=readRegistre(spi,cs,_LSM9DS0_REGISTER_OUT_X_H_A)
    x=convertirData( int.from_bytes(bufH,"big"), int.from_bytes(bufL,"big"))

    bufL=readRegistre(spi,cs,_LSM9DS0_REGISTER_OUT_Y_L_A)
    bufH=readRegistre(spi,cs,_LSM9DS0_REGISTER_OUT_Y_H_A)
    y=convertirData( int.from_bytes(bufH,"big"), int.from_bytes(bufL,"big"))

    bufL=readRegistre(spi,cs,_LSM9DS0_REGISTER_OUT_Z_L_A)
    bufH=readRegistre(spi,cs,_LSM9DS0_REGISTER_OUT_Z_H_A)
    z=convertirData( int.from_bytes(bufH,"big"), int.from_bytes(bufL,"big"))

    return x,y,z

#lecture donnees magnetometre
def readMagneto(spi,cs):
    bufL=readRegistre(spi,cs,_LSM9DS0_REGISTER_OUT_X_L_M)
    bufH=readRegistre(spi,cs,_LSM9DS0_REGISTER_OUT_X_H_M)
    x=convertirData( int.from_bytes(bufH,"big"), int.from_bytes(bufL,"big"))

    bufL=readRegistre(spi,cs,_LSM9DS0_REGISTER_OUT_Y_L_M)
    bufH=readRegistre(spi,cs,_LSM9DS0_REGISTER_OUT_Y_H_M)
    y=convertirData( int.from_bytes(bufH,"big"), int.from_bytes(bufL,"big"))

    bufL=readRegistre(spi,cs,_LSM9DS0_REGISTER_OUT_Z_L_M)
    bufH=readRegistre(spi,cs,_LSM9DS0_REGISTER_OUT_Z_H_M)
    z=convertirData( int.from_bytes(bufH,"big"), int.from_bytes(bufL,"big"))

    return x,y,z

####################################################################
#                   CONVERSION DES DONNEES
####################################################################

#La conversion est directement appellée dans readCapteur()
#valeur codee en complement a 2
def convertirData(high,low):
    result = high * 256 + low
    if result > 32767 :
    	result = result - 65536
    return result
